#include<bnode.h>

/*
 * si noti il doppio asterisco
 * pu essere necessario modificare root
 */
void insert(node ** tree, int val)
{
	node *root = *tree;
	if(!root) {
		root = malloc(sizeof(node));
		root->left = root->right = NULL;
		root->data = val;
		*tree = root;
		return;
	}

	/*
	 * non sono ammessi dati duplicati
	 * ma non  obbligatorio
	 *
	 * una delle possibili alternative potrebbe essere gestire
	 * un contatore: if(val == root->data) { root->counter++; return; }
	 *
	 * se si ammettono duplicazioni basta semplicemente eliminare l'istruzione
	 */
	if(val == root->data) return;
	if(val < root->data) {
		insert(&root->left, val);
	} else {
		insert(&root->right, val);
	}
}

void print_preorder(node * tree)
{
	if (tree) {
		printf("%d ",tree->data);
		print_preorder(tree->left);
		print_preorder(tree->right);
	}
}

void print_inorder(node * tree)
{
	if (tree) {
		print_inorder(tree->left);
		printf("%d ",tree->data);
		print_inorder(tree->right);
	}
}

void print_postorder(node * tree)
{
	if (tree) {
		print_postorder(tree->left);
		print_postorder(tree->right);
		printf("%d ",tree->data);
	}
}

void deltree(node * tree)
{
	if (tree) {
		deltree(tree->left);
		deltree(tree->right);
		free(tree);
	}
}

node* search(node * tree, int val)
{
	if(!tree) return NULL;

	if(val == tree->data) return tree;
	if(val < tree->data) {
		return search(tree->left, val);
	} else {
		return search(tree->right, val);
	}
}

void delete(node ** ptree, int val)
{
	node *tree = *ptree;
	if(!tree) return;

	if(val == tree->data) {
		if(tree->left==NULL) {
			*ptree = tree->right;
			free(tree);
			return;
		}
		if(tree->right==NULL) {
			*ptree = tree->left;
			free(tree);
			return;
		}
#ifdef BILANCIATO
		if(count(tree->right) > count(tree->left)) {
			node *tmp = tree->right;
			while(tmp->left) tmp = tmp->left;
			tree->data = tmp->data;
			delete(&tree->right, tmp->data);
		} else {
			node *tmp = tree->left;
			while(tmp->right) tmp = tmp->right;
			tree->data = tmp->data;
			delete(&tree->left, tmp->data);
		}
#else
			node *tmp = tree->right;
			while(tmp->left) tmp = tmp->left;
			tree->data = tmp->data;
			delete(&tree->right, tmp->data);
#endif
		return;
	}
	if(val < tree->data) {
		delete(&tree->left, val);
	} else {
		delete(&tree->right, val);
	}
}

int count(node *tree)
{
	if(!tree) return 0;
	return 1 + count(tree->left) + count(tree->right);
}

/*
 * se il vettore  ordinato l'albero  bilanciato
 */
node* array_to_tree(int *source, int n)
{
	if(n<=0) return NULL;
	node* root = malloc(sizeof(node));
	// la prima parte del vettore  il sottoalbero sinistro
	root->left = array_to_tree(source, n/2);
	// la parte finale del vettore  il sottoalbero destro
	root->right = array_to_tree(source+n/2+1, n - n/2 - 1);
	// il valore mediano  la radice
	root->data = source[n/2];
	return root;
}

/*
 * l'albero  visitato "per livelli"
 * viene prodotto un vettore di puntatori
 */
node ** tree_to_array(node *tree)
{
	int n = count(tree), i, j;
	// n+1 per lasciare un ultimo puntatore nullo
	node **out = calloc(n+1, sizeof(node *));
	out[0] = tree;
	for(i=0, j=1; i<j; i++) {
		if(out[i]->left) out[j++]=out[i]->left;
		if(out[i]->right) out[j++]=out[i]->right;
	}
	return out;
}
